// Define the Chart interface with the necessary properties
interface Chart {
    publicUrl: string;
    id: string;
    metadata: {
        publish: {
            "embed-height": string;
        };
    };
    title: string;
}

// Define the configuration interface
interface DataWrapperConfig {
    allowEditing: boolean;
    dark: (e: string) => boolean | string;
    dev: boolean;
    fitchart: boolean;
    logo: string;
    logoId: string;
    map2svg: boolean;
    plain: boolean;
    previewId: string;
    search: string;
    static: boolean;
    svgonly: boolean;
    theme: string;
    transparent: boolean;
    webComponentJsHash?: string;
}

// Set of falsy values
const t = new Set(["0", "false", "null"]);

// Default configuration object
const e: DataWrapperConfig = {
    allowEditing: false,
    dark: function (e: string): boolean | string {
        return "auto" === e ? e : !!e && !t.has(e);
    },
    dev: false,
    fitchart: false,
    logo: "",
    logoId: "",
    map2svg: false,
    plain: false,
    previewId: "",
    search: "",
    static: false,
    svgonly: false,
    theme: "",
    transparent: false,
};

// Function to parse data attributes
function parseDataAttributes(element: Element): DataWrapperConfig {
    return (function (getAttribute: (attr: string) => string | null): DataWrapperConfig {
        return Object.fromEntries(
            Object.entries(e).map(([key, val]) => {
                const attrVal = getAttribute(`data-${key}`);
                return val === Boolean
                    ? [key, !!attrVal && !t.has(attrVal)]
                    : [key, val(attrVal)];
            })
        ) as DataWrapperConfig;
    })(attr => element.getAttribute(attr));
}

// Function to render the datawrapper
export function renderDatawrapper(): void{
    console.log("Attempting to run renderDatawrapper");

    if (typeof window === 'undefined') {
        console.warn('renderDatawrapper is running in a non-browser environment.');
        return;
    }

    console.log("Running renderDatawrapper in a browser environment");

    console.log("Called function renderDatawrapper")
    if (window.datawrapper && window.datawrapper.render) {
        console.log("true: (window.datawrapper && window.datawrapper.render) "); 
    }
    if (window.datawrapper && window.datawrapper.render) return;
   
    window.datawrapper = {
        chartData: {},
        libraries: {},
        WebComponent: {},
        render: async function (t: {
            origin: any;
            chart: Chart;
            dependencies: string[];
            chartAutoDark: boolean;
            script?: HTMLScriptElement | undefined;
            webComponentJsHash?: string;
        }, e: { [key: string]: any } = {}): Promise<void> {
            t.script = document.currentScript as HTMLScriptElement;
            
            if (!t.chart.publicUrl && t.script) {
                t.chart.publicUrl = t.script.src.replace(/\/(embed\.js)?$/, "/");
                console.log("(!t.chart.publicUrl && t.script) true"); 
            }
            const origin = e.origin || (t.chart.publicUrl || "").replace(/\/(index\.html)?$/, "");
            t.origin = origin;

            window.datawrapper.chartData[t.chart.id] = Promise.resolve(t);
            const targetId = `datawrapper-vis-${t.chart.id}`;
            const target = e.target || document.createElement("div");
            target.classList.add("datawrapper-script-embed");
            target.setAttribute("id", targetId);
            if (!e.target) {
                console.log("true: (!e.target)")
                t.script.parentNode?.insertBefore(target, t.script);
            }
            target.style.all = "initial";
            const renderFlags = e.flags || parseDataAttributes(t.script);
            const isAutoDark = renderFlags.dark === "auto" || (renderFlags.dark === null && t.chartAutoDark);
            if (isAutoDark) {
                console.log("true: (isAutoDark) ")
                renderFlags.dark = window.matchMedia && window.matchMedia("(prefers-color-scheme: dark)").matches;
            }
            if (!("customElements" in window) || !("ResizeObserver" in window)) {
                console.log("true: (!(\"customElements\" in window)"); 
                const iframe = document.createElement("iframe");
                iframe.src = t.chart.publicUrl;
                iframe.setAttribute("title", t.chart.title);
                iframe.setAttribute("scrolling", "no");
                iframe.setAttribute("frameborder", "0");
                iframe.setAttribute("style", "width: 0; min-width: 100% !important; border: none;");
                iframe.setAttribute("height", t.chart.metadata.publish["embed-height"]);
                iframe.setAttribute("data-external", "1");
                target.appendChild(iframe);

                // Cast window to any to access addEventListener
                (window as any).addEventListener("message", function (event: MessageEvent) {
                    if (event.data["datawrapper-height"] !== undefined) {
                        console.log("true (event.data[\"datawrapper-height\"] !== undefined)");
                        const iframes = document.querySelectorAll("iframe");
                        for (const key in event.data["datawrapper-height"]) {
                            for (let i = 0; i < iframes.length; i++) {
                                if (iframes[i].contentWindow === event.source) {
                                    console.log("rrue: (iframes[i].contentWindow === event.source)"); 
                                    const height = event.data["datawrapper-height"][key] + "px";
                                    iframes[i].style.height = height;
                                }
                            }
                        }
                    }
                });
                return;
            }

            let componentInstance: any = null;
            const componentProps = {
                target: target,
                props: {
                    outerContainer: target,
                    dependencyPromises: window.datawrapper.dependencyPromises,
                    renderFlags: renderFlags,
                    isAutoDark: isAutoDark,
                    ...t,
                    selfDestruct() {
                        componentInstance && componentInstance.$destroy ? componentInstance.$destroy() : target.innerHTML = "";
                    }
                },
                hydrate: false
            };

            const loadScript = (url: string): Promise<void> => {
                return new Promise<void>((resolve, reject) => {
                    const script = document.createElement("script");
                    script.src = url;
                    script.charset = "utf-8";
                    script.onload = () => resolve();
                    script.onerror = reject;
                    document.body.appendChild(script);
                });
            };

            const loadDependency = (dep: string): Promise<void> => {
                const scriptUrl = dep.startsWith("http") ? dep : `${origin}/${dep}`;
                return window.datawrapper.libraries[scriptUrl] || (window.datawrapper.libraries[scriptUrl] = loadScript(scriptUrl));
            };

            const [mainDep, visDep, ...otherDeps] = t.dependencies;
            const mainVisLoaded = Promise.all([loadDependency(mainDep), loadDependency(visDep)]);

            const loadComponents = async (): Promise<void> => {
                for (const dep of otherDeps) {
                    await loadDependency(dep);
                }
                const webComponentJsHash = t.webComponentJsHash ?? "";
                const WebComponent = window.datawrapper.WebComponent[webComponentJsHash] || window.datawrapper.VisualizationWebComponent;
                const customElementTag = customElements.get("datawrapper-visualization") && customElements.get("datawrapper-visualization") !== WebComponent ? `datawrapper-visualization-${webComponentJsHash}` : "datawrapper-visualization";
                if (!customElements.get(customElementTag)) {
                    customElements.define(customElementTag, WebComponent);
                }
                const WebComponentClass = customElements.get(customElementTag);
                if (WebComponentClass) {
                    componentInstance = new WebComponentClass(componentProps);
                } else {
                    console.error(`Web component class '${customElementTag}' not found.`);
                }
            };

            await mainVisLoaded;
            await loadComponents();
        }
    };
}

// Export the function so it can be called explicitly
window.renderDatawrapper = renderDatawrapper; 

// Call the render function when the window loads
window.onload = function () {
    renderDatawrapper(); 
    console.log("Datawrapper deployed")
};




